/*
 * This source code is a C++ implementation of task swapping graphs
 * Copyright (C) 2011, Dohan Kim <dkim@airesearch.kr>
 * Author : Dohan Kim < dkim@airesearch.kr >
 *
 * File Name : PGroup.h
 * Description : Header file of Permutation Groups
 *
 */

#ifndef _PGROUP_H
#define _PGROUP_H

#include <vector>
#include <iostream>
#include <sstream>
#include <string>
#include <iomanip>

using namespace std;

typedef pair<int, int> transposition;

class PGroup
{
  private:
  public:
	PGroup();
	~PGroup();
	int *inverse( int *perm, int length );
	int *multiplication( int *perm1, int *perm2, int length );
	int *stable_displacement_vector( int *perm, int length );
	int perm_element_find( int *perm, int length, int k );
	int line_inversion_number( int *perm, int length );
	int ring_inversion_number( int *dvec, int length );
	vector< vector<int> > disjoint_cycles( int *perm, int length );
	vector< vector<int> > canonical_disjoint_cycles( int bipartite_index, 
		vector<char> &cycle_type, vector< vector<int> > &cycle );
	int bipartite_internal_cycle_factorization( int bipartite_index, 
		vector<transposition> &complete_bipartite_sequence, vector<int> &cycle );
	int bipartite_external_cycle_factorization( int bipartite_index, 
		vector<transposition> &complete_bipartite_sequence, vector<int> &cycle );
	int bipartite_simple_cycle_factorization( int bipartite_index, 
		vector<transposition> &complete_bipartite_sequence, vector<int> &cycle );
	int bipartite_mixed_cycle_factorization( int bipartite_index, 
		vector<transposition> &complete_bipartite_sequence, vector<int> &cycle );
};

#endif /* _PGROUP_H */
