/*
 * This source code is a C++ implementation of task swapping graphs.
 * Copyright (C) 2011, Dohan Kim <dkim@airesearch.kr>
 * Author : Dohan Kim < dkim@airesearch.kr >
 *
 * File Name : TSG.h
 * Description : Header file of TSG
 *
 */

#ifndef _TSG_H
#define _TSG_H

#include <vector>
#include <iostream>
#include <sstream>
#include <string>
#include <iomanip>

#include <math.h>

#include "PGroup.h"

using namespace std;

typedef struct
{
	char *input_file;
	char topology_type;
	int number_of_tasks;
	int swapping_distance;
	int bipartite_index;
	bool random_assignments;
	int *source_task_assignment;
	int *target_task_assignment;
} TSGIO;

class TSG
{
  private:
	TSGIO *tsgIO;
	PGroup *pgroup;
  public:
	TSG( TSGIO *tsgIO, PGroup *pgroup );
	~TSG(){};
	int print(); 
	int tsg_line(); 
	int tsg_star();
	int tsg_complete();
	int tsg_ring();
	int tsg_complete_bipartite();
	int tsg_print_result( vector<transposition> &minimum_length_sequence, int upper_bound );
	int perm_ring_sorting( int *perm_for_sorting, int *d_vec, int number_of_tasks, vector<int> &ring_sequence );
	vector<int> perm_bubble_sorting( int *perm, int length );
};

extern TSGIO tsgIO;
extern int tsg_print( TSGIO *tsgIO );
extern int tsg_initialize( TSGIO *tsgIO );
#endif /* _TSG_H */
