/*
 * This source code is a C++ implementation of task swapping graphs.
 * Copyright (C) 2011, Dohan Kim <dkim@airesearch.kr>
 * Author : Dohan Kim < dkim@airesearch.kr >
 *
 * File Name : main.cpp 
 * Description : main file of task swapping graphs
 *
 */

#include "TSG.h"
#include "PGroup.h"

#include <cstdlib>
#include <cstring>
#include <ctime>
#include <algorithm>
#include <fstream>
#include <iostream>

using namespace std;
TSGIO tsgIO;

int tsg_initialize( TSGIO *tsgIO )
{
	char line[1024];
	char buf[1024];
	vector<string> temp_list;
	FILE *fptr1 = NULL;

	if( ( fptr1 = fopen( tsgIO->input_file, "r" ) ) == NULL )
	{
		cout << "FILE OPEN ERROR: "<<__FILE__<<","<<__LINE__<<endl;
		exit( 1 );
	}
	else
	{
		while( fgets( line, sizeof( line ), fptr1 ) != NULL )
		{
			size_t i = strspn( line, " \t\n\v" );

			if ( line[i] == '#' )
			{
				continue;
			}
			else if( !strncmp( line, "TopologyType", 12 ) )
			{
				memset( buf, '\0' , sizeof( buf) );
				sscanf( line,"%*s %s", buf );
				tsgIO->topology_type = buf[0];
			}
			else if( !strncmp( line, "NumberOfTasks", 13 ) )
			{
				memset( buf, '\0' , sizeof( buf) );
				sscanf( line,"%*s %s", buf );
				tsgIO->number_of_tasks = atoi( buf );

				tsgIO->source_task_assignment =
					( int * )malloc( sizeof( int ) * tsgIO->number_of_tasks );
				tsgIO->target_task_assignment =
					( int * )malloc( sizeof( int ) * tsgIO->number_of_tasks );
			}
			else if( !strncmp( line, "RandomAssignments", 17 ) )
			{
				memset( buf, '\0' , sizeof( buf) );
				sscanf( line,"%*s %s", buf );

				if( buf[0] == 'Y' )
				{
					tsgIO->random_assignments = true;
				}
				else
				{
					tsgIO->random_assignments = false;
				}
			}
			else if( !strncmp( line, "BipartiteIndex", 14 ) )
			{
				memset( buf, '\0' , sizeof( buf) );
				sscanf( line,"%*s %s", buf );
				tsgIO->bipartite_index = atoi( buf );
			}
			else if( !strncmp( line, "SwappingDistance", 16 ) )
			{
				memset( buf, '\0' , sizeof( buf) );
				sscanf( line,"%*s %s", buf );
				tsgIO->swapping_distance = atoi( buf );
			}
			else if( !strncmp( line, "SourceTaskAssignment", 20 ) )
			{
				if( tsgIO->random_assignments )
				{
					break;
				}

				char *temp_ptr = NULL;
				int index = 0;
				memset( buf, '\0' , sizeof( buf ) );
				strncpy( buf, line, strlen( line ) );

				temp_ptr = strtok( buf, "\t ");

				while( temp_ptr != NULL && index < tsgIO->number_of_tasks )
				{
					temp_ptr = strtok( NULL, "\t " );
					tsgIO->source_task_assignment[index++] = atoi( temp_ptr );
				}
			}
			else if( !strncmp( line, "TargetTaskAssignment", 20 ) )
			{
				if( tsgIO->random_assignments )
				{
					break;
				}

				char *temp_ptr = NULL;
				int index = 0;
				memset( buf, '\0' , sizeof( buf ) );
				strncpy( buf, line, strlen( line ) );

				temp_ptr = strtok( buf, "\t ");

				while( temp_ptr != NULL && index < tsgIO->number_of_tasks )
				{
					temp_ptr = strtok( NULL, "\t " );
					tsgIO->target_task_assignment[index++] = atoi( temp_ptr );
				}
			}
			else
			{
				continue;

			}
		}
	}

	fclose( fptr1 );

	return 1;
}

int tsg_print( TSGIO *tsgIO )
{
	cout<<"************************************************"<<endl;
	cout<<"Topology Type: ";

	if( tsgIO->topology_type == 'L' )
	{
		cout<<"Line Topology";
	}
	else if( tsgIO->topology_type == 'S' )
	{
		cout<<"Star Topology";
	}
	else if( tsgIO->topology_type == 'C' )
	{
		cout<<"Complete Topology";
	}
	else if( tsgIO->topology_type == 'B' )
	{
		cout<<"Complete Bipartite Topology";
	}
	else if( tsgIO->topology_type == 'R' )
	{
		cout<<"Ring Topology";
	}
	else
	{
		//cannot be reached here
		exit(-1);
	}

	cout<<endl;

	cout<<"Number of Tasks: "<<tsgIO->number_of_tasks<<endl;

	if( tsgIO->topology_type == 'B' )
	{
		cout<<"Bipartite Index: "<<tsgIO->bipartite_index<<endl;
	}

	if( tsgIO->random_assignments == true )
	{
		cout<<"Random Initial Assignments: Yes"<<endl;
	}
	else
	{
		cout<<"Random Initial Assignments: No"<<endl;
	}

	cout<<"Source Task Assignment: ";
	for( int i = 0 ; i < tsgIO->number_of_tasks ; i++ )
	{
		cout<<tsgIO->source_task_assignment[i]<<" ";
	}
	cout<<endl;

	cout<<"Target Task Assignment: ";
	for( int j = 0 ; j < tsgIO->number_of_tasks ; j++ )
	{
		cout<<tsgIO->target_task_assignment[j]<<" "; 
	}
	cout<<endl;

	fflush( stdout );
	return 1;
}

int main( int argc, char **argv )
{
	if( argc < 2 )
	{
		cout<<"Usage: ./tsg input_file"<<endl;
		exit( 1 );
	}

	srand ( unsigned ( time (NULL) ) );

	PGroup pgroup;
	tsgIO.input_file = argv[1];
	tsg_initialize( &tsgIO );

	if( tsgIO.random_assignments )
	{
		for( int i = 0; i < tsgIO.number_of_tasks; i++ )
		{
			tsgIO.source_task_assignment[i] = i + 1;
			tsgIO.target_task_assignment[i] = i + 1;
		}

		random_shuffle( tsgIO.source_task_assignment, tsgIO.source_task_assignment + tsgIO.number_of_tasks );
		random_shuffle( tsgIO.target_task_assignment, tsgIO.target_task_assignment + tsgIO.number_of_tasks );
	}

	TSG tsg( &tsgIO, &pgroup );
	tsg_print( &tsgIO );

	if( tsgIO.topology_type == 'L' )
	{
		tsg.tsg_line();
	}
	else if( tsgIO.topology_type == 'S' )
	{
		tsg.tsg_star();
	}
	else if( tsgIO.topology_type == 'C' )
	{
		tsg.tsg_complete();
	} 
	else if( tsgIO.topology_type == 'R' )
	{
		tsg.tsg_ring();
	}
	else if( tsgIO.topology_type == 'B' )
	{
		tsg.tsg_complete_bipartite();
	}

	tsg.print();

	return 1;
}
